/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};
var saveAsFilePath; // undefined initially
var regFileFormat = "CSV";

var tempScale = "C";        // Holds current temperature Scale
var sampleIndex = 0;        // Current Index of Sample to be placed in array
var sampleTimeArray = [];   // Array for time-samples

/****************************** Start of file stream code *******************************/
var mystream;
var mywriter;
var myTextEnc = new TextEncoder();
var saveStart = function(filename) {
	if (!filename) filename = 'Temperature_Data_Stream.csv';
	mystream = streamSaver.createWriteStream(filename);
	saveAbort();
	mywriter = mystream.getWriter();
	saveData(myTextEnc.encode('Time, Temperature (' + tempScale + ')\n'));
	templateObj.showToastMsg( "check" , "Streaming temperature chart data to file.", 4);
};
var saveStop = function() {
	if (mywriter) {
		mywriter.close();
		mywriter = null;
	    templateObj.showToastMsg( "check" , "File stream closed", 4);
	}
};
var saveAbort = function() {
	if (mywriter){
		mywriter.abort('reason');
		mywriter = null;
	}
};
var saveData = function(data) {
	if (mywriter) {
		mywriter.write(data);
	}
};
window.addEventListener('unload', (event) => {
	//  console.log('unload', event);
	saveAbort(); // seems ok but please test.
});
var saveDummyData = function() {
	// To save textual data use TextEncoder to encode the data first
	var my_text = 'abcdef 01234 ABCDEF 56789 some text to go ... done';
	var myary = myTextEnc.encode(my_text + '\n');
	saveData(myary);
	// To save byte array data like below, no need to encode it.
	//saveData([1,2,3,4,5,6,7,8,9,10, 100,101]);  
};
/****************************** End of file stream code *********************************/

var convertTemp = function(value, isToCelsius) {
    var result = value;
    if (isToCelsius) {
        result = (value - 32) / 1.8;
    } else {
        result = (value * 1.8) + 32;
    }
    return Number.parseFloat(result).toFixed(3);
};

var getF = function(value){
	return Number.parseFloat((value * 1.8) + 32).toFixed(3);
};

var getC = function(value){
    return Number.parseFloat((value - 32) / 1.8).toFixed(3);
};

function convertTimestamp(timestamp) {
    var d = new Date(timestamp), // Convert the passed timestamp to milliseconds
        h = d.getHours(),
        min = ('0' + d.getMinutes()).slice(-2), // Add leading 0.
        sec= ('0' + d.getSeconds()).slice(-2), // Add leading 0.
        msec= ('0' + d.getMilliseconds()).slice(-3), // Add leading 0.
        time;
    time = h + ':' + min + ':' + sec + '.' + msec + ' ';
    return time;
}

var saveText = function(fname, text) {
    var blob = new Blob([text], {
        type: 'text/plain;charset=utf-8'
    });
    saveAs(blob, fname);
    return false;
};

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/
	
	//Set I2C Address
	gc.databind.registry.bind('ti_widget_droplist1_i2c_address.selectedValue','reg.$deviceAddrs');
	
	// Temperature Result conversion
    gc.databind.registry.bind('widget.ti_widget_label_dataTemp.label','ti_model_register.TEMP_RESULT.TEMP',
        function(value) {
            if (tempScale === "C")
				return Number.parseFloat(value).toFixed(3);
			else
				return Number.parseFloat(getF(value)).toFixed(3);
        }
    );
    
    // Temperature High Threshold
    gc.databind.registry.bind('widget.ti_widget_label9_high_temp_scale.label','widget.ti_widget_numberbox_high_temp_scale.value',
        function(value){
            if (tempScale === "C")
                return Number.parseFloat(getF(value)).toFixed(3);
		    else 
			    return Number.parseFloat(getC(value)).toFixed(3); 
        },
        function(value){
            if (tempScale === "C")
		        return Number.parseFloat(getC(value)).toFixed(3);
	        else 
                return Number.parseFloat(getF(value)).toFixed(3);
        }
    );
    gc.databind.registry.bind('ti_model_register.HIGH_LIMIT.HIGH_LIMIT','widget.ti_widget_label9_high_temp_scale.label',
        function(value){
            if (tempScale === "C")
			    return Number.parseFloat(getC(value)).toFixed(3);
		    else 
                return Number.parseFloat(value).toFixed(3);
        },
        function(value){
            if (tempScale === "C")
			    return Number.parseFloat(getF(value)).toFixed(3);
		    else 
                return Number.parseFloat(value).toFixed(3);
        }
    );
    
    // Temperature Low Threshold
    gc.databind.registry.bind('widget.ti_widget_label11_low_temp_scale.label','widget.ti_widget_numberbox_low_temp_scale.value',
        function(value){
            if (tempScale === "C")
		        return Number.parseFloat(getF(value)).toFixed(3);
		    else 
			    return Number.parseFloat(getC(value)).toFixed(3);
        },
        function(value){
            if (tempScale === "C")
		        return Number.parseFloat(getC(value)).toFixed(3);
	        else 
                return Number.parseFloat(getF(value)).toFixed(3);
        }
    );
    gc.databind.registry.bind('ti_model_register.LOW_LIMIT.LOW_LIMIT','widget.ti_widget_label11_low_temp_scale.label',
        function(value){
            if (tempScale === "C")
			    return Number.parseFloat(getC(value)).toFixed(3);
		    else 
                return Number.parseFloat(value).toFixed(3);
        },
        function(value){
            if (tempScale === "C")
			    return Number.parseFloat(getF(value)).toFixed(3);
		    else 
                return Number.parseFloat(value).toFixed(3);
        }
    );
    
    // Bind conversion mode value to conversion mode register
    gc.databind.registry.bind('widget.ti_widget_droplist_conversion_mode.selectedValue','ti_model_register.CONFIG_REG.SD');
    
    // Bind conversion rate value to conversion rate register
    gc.databind.registry.bind('widget.ti_widget_droplist_conversion_rate.selectedValue','ti_model_register.CONFIG_REG.CONVERSION_RATE');
    
    // Bind fault limit value to fault limit register
    gc.databind.registry.bind('widget.ti_widget_droplist_fault_limit.selectedValue','ti_model_register.CONFIG_REG.FAULT_QUEUE');
    
    // Binding of auto-read on settings tab to register page tab
	gc.databind.registry.bind('ti_widget_droplist2_graph_refresh_rate.selectedIndex', 'ti_widget_register_page.autoReadValue', 
		function(value){                                // Getter function, sets drop list based on autoReadValue changes
			switch(Number(value)){
			case -1: //Auto read is off
				return 0;
			case 0: // As Fast as possible
				return 7;
			case 1000: // 1s delay
				return 1;
			case 5000: // 5s delay
				return 2;
			case 10000: // 10s delay
				return 3;
			case 20000: // 20s delay
				return 4;
			case 30000: // 30s delay
				return 5;
			case 60000: // 60s delay
				return 6;
			default:
				return 7;
			}
		},
		function(value){                                // Setter function, sets autoReadValue based on droplist chnages
			switch(Number(value)){
			case 0: //Auto read is off
				return -1;
			case 1: // 1s delay
				return 1000;
			case 2: // 5s delay
				return 5000;
			case 3: // 10s delay
				return 10000;
			case 4: // 20s delay
				return 20000;
			case 5: // 30s delay
				return 30000;
			case 6: // 60s delay
				return 60000;
			case 7: // As Fast as possible
				return 0;
			default:
				return 0;
		    }
	    });
	
	// Bind alert pin polarity to polarity register
    gc.databind.registry.bind('widget.ti_widget_droplist1_pin_polarity.selectedValue','ti_model_register.CONFIG_REG.POL');
    
    // Bind send one shot button to conversion mode
	gc.databind.registry.bind('widget.ti_widget_button_one_shot.disabled','ti_model_register.CONFIG_REG.SD',
        function(value) {
            // convert register value to object
            var result = !value;
            return result;
        }
    );
    
    // Binding of auto-read on data capture tab to settings tab
	gc.databind.registry.bind('widget.ti_widget_droplist_graph_refresh_rate.selectedIndex', 'ti_widget_droplist2_graph_refresh_rate.selectedIndex');
	
	// Bind red LED to alert
	gc.databind.registry.bind('widget.ti_tile_led_alert.value', 'ti_model_register.u2a.gpio.REDLED',
	    function(value){
	        return !value;
	    }
	);
	
	// Bind thermostat mode to thermostat mode register
    gc.databind.registry.bind('widget.ti_widget_droplist_thermo_mode.selectedValue','ti_model_register.CONFIG_REG.THERMOSTAT_MODE');
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('set_csv_format', function() {
        regFileFormat = "CSV";
        gc.nav.setActionChecked('set_csv_format', true);
        gc.nav.setActionChecked('set_json_format', false);
    }, function() {
        return  true; // isAvailable
    }, function() {
        return true; // isVisible
    });
    gc.nav.setActionChecked('set_csv_format', true);

    gc.nav.registerAction('set_json_format', function() {
        regFileFormat = "JSON";
        gc.nav.setActionChecked('set_csv_format', false);
        gc.nav.setActionChecked('set_json_format', true);
    }, function() {
        return true; // isAvailable
    }, function() {
        return true;  // isVisible
    });

    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    };

    var getCsvForRegs = function(){
        var result = "register,address,value\n";
        var regPage = document.querySelector('#ti_widget_register_page');
        var regBlocks = regPage && regPage.regBlocks;
        if (regBlocks) {
            for (var b=0; b < regBlocks.length; b++) {
                var regBlock = regBlocks[b];
                for (var r=0; r < regBlock.registers.length; r++) {
                    var reg = regBlock.registers[r];
                    result += reg.name+","+reg.addr+","+reg.value+"\n";
                }
            }
        }
        return result;
    };

    /*
     * saveRegs saves the register values from the register page into either a CSV file 
     * format or a stringified JSON object and saves them to a file.  If isSaveAs
     * parameter is true (or the saveAsFilePath global has not been set), 
     * it shows the 'browseAndSave' dialog to allow the user to 
     * specify which folder to save the file into, otherwise it saves the contents to 
     * the file path specified by saveAsFilePath
     */
    var saveRegs = function(calledBy, isSaveAs) {
        var fileFilter;
        var regData;
        switch (regFileFormat) {
            case "CSV":
                regData = getCsvForRegs();
                fileFilter = ".csv";
                // make sure the file extension matches the regFileFormat
                if (saveAsFilePath) {
                    saveAsFilePath = saveAsFilePath.replace(".json", ".csv");
                }
                break;
            case "JSON":
            default:
                fileFilter = ".json";
                regData = JSON.stringify(templateObj.getRegisterData());
                // make sure the file extension matches the regFileFormat
                if (saveAsFilePath) {
                    saveAsFilePath = saveAsFilePath.replace(".csv", ".json");
                }
        }
        var fileName = saveAsFilePath;
        if (!fileName) {
            fileName = "regs" + fileFilter;
        }
        if (gc.desktop.isDesktop()) {
            if (isSaveAs || !saveAsFilePath ) {
                gc.File.browseAndSave(regData, fileName, fileFilter, function (result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", calledBy + " " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        saveAsFilePath = result.localPath;
                        gc.nav.enableAction('save_registers');
                        templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + result.localPath);
                    }
                });
            } else {
                gc.File.save(regData, {"localPath": saveAsFilePath}, null, function(errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                    } else {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + saveAsFilePath);
                    }
                });
            }
        } else {
            saveText(fileName, regData, 'text/plain;charset=utf-8');
            templateObj.$.ti_widget_eventlog_view.log("info", calledBy+": "+fileName+" file downloaded");
        }
    };

    /*
     * getRegsFromCsv decodes the string passed in as a parameter as the contents of a CSV file
     * and updates the registers in the regBlocks object with the values from the CSV file.
     * NOTE that the target is not updated with these values by this function.  To update the
     * target registers, please call setRegisterData.
     */
    getRegsFromCsv = function(csvContents) {
        var lines = csvContents.split('\n');
        lines.shift(); // discard the title row
        // create a map of the csv values
        var csvMap = new Map();
        for (var i=0; i < lines.length; i++){
            var regInfo = lines[i].split(",");
            var name = regInfo[0].trim().toLowerCase();
            var addr = +regInfo[1];
            var value = +regInfo[2];
            var key = name+"@"+addr;
            csvMap.set(key,value);
        }
        var regPage = document.querySelector('#ti_widget_register_page');
        var regBlocks = regPage && regPage.regBlocks;
        if (regBlocks) {
            for (var b = 0; b < regBlocks.length; b++) {
                var regBlock = regBlocks[b];
                for (var r = 0; r < regBlock.registers.length; r++) {
                    var reg = regBlock.registers[r];
                    var regAdrs = +reg.addr;
                    var regKey = reg.name.trim().toLowerCase() + "@"+regAdrs;
                    if (csvMap.has(regKey)){
                        reg.value = csvMap.get(regKey);
                    }
                }
            }
        }
        return regBlocks;
    };

    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    var fileType =  regFileFormat;
                    var dotIndex = (properties && properties.name) ? properties.name.lastIndexOf(".") : -1;
                    if (dotIndex >= 0 && dotIndex < properties.name.length) {
                        fileType = properties.name.substring(dotIndex + 1).toUpperCase();
                        if (fileType !== "JSON" && fileType !== "CSV") {
                            fileType = regFileFormat;
                        }
                    }
                    switch (fileType) {
                        case "CSV":
                            regData = getRegsFromCsv(contents);
                            break;
                        default:
                        case "JSON":
                            regData = JSON.parse(contents);
                            if (typeof regData === "string") {
                                regData = JSON.parse(regData);
                            }
                            if (regData && regData.data) {
                                regData = regData.data;
                            }
                            break;
                    }
                    if (regData) {
                        if (templateObj.setRegisterData(regData)) {
                            templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                        }
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse " + regFileFormat + " file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            saveRegs("Save Registers", false);
        }
    }, function() {
        return saveAsFilePath;  // isAvailable
    }, function() {
        return saveAsFilePath;  // isVisible
    });
    gc.nav.disableAction('save_registers');
    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            saveRegs("Save Registers As", true);
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_log_pane', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_eventlog_view.openView();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_scripting_window', function() {
        window.open('app/scripting.html', '_evm_scripting');
    }, function() {
        return true;
    }, function() {
        return true;
    });
});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            console.log("Application template has been stamped.");
            templateObj.$.ti_widget_toast.hideToast();
            templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
            gc.nav.registerChecklistActions('fileFormat.', fileFormat,
                {
                    onSelectionHandler: function(formatName, detail) {
                        regFileFormat = formatName;
                        gc.localStorage.setItem('fileFormat', formatName);
                    },

                    onIsAvailableHandler: function() {
                        return false;
                    }
                }
                , gc.localStorage.getItem('fileFormat') || regFileFormat, false);
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
            if (registerModel) {
                registerModel.addEventListener("usbHidModelReceivingData", function (event) {
                    templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
                });
            } else {
                console.log("no ti-model-register component found in this application!");
            }

            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

            // open the eventlog viewer when the user clicks on the book icon in the status bar:
            templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
            templateObj.$.ti_widget_statusbar.addEventListener("status-icon-clicked", function(event) {
                // TODO: add whatever actions you wish to perform in response to the appStatusIcon in the statusbar
                // being clicked.  See event.details for info on the icon and app status
            });

            // Add custom events below //////////////////////////////////////////////////////////////////////////////////////////////
            // Set I2C Frequency based on droplist
            templateObj.$.ti_widget_droplist_i2c_frequency.addEventListener("selected", function(event) {
               var u2a= document.querySelector('ti-model-register').getCodec();         // get codec
               var Cmd_I2C_Set = 1;                                                     // Command to set I2C port
               var speed = templateObj.$.ti_widget_droplist_i2c_frequency.selectedIndex;    // Speed based on droplist index: 0:100kHz, 1:400kHz
               var commandSent = u2a.sendCommandPacket(Cmd_I2C_Set, [speed, 0, 1])      // send command to set I2C, see packet definition in USB2ANY.h
            });
            
            // Update temp scales
            var tempVal = gc.databind.registry.getBinding('ti_model_register.TEMP_RESULT.TEMP').getValue();
            var highThreshold = gc.databind.registry.getBinding('ti_model_register.HIGH_LIMIT.HIGH_LIMIT').getValue();
            var lowThreshold = gc.databind.registry.getBinding('ti_model_register.LOW_LIMIT.LOW_LIMIT').getValue();
            templateObj.$.ti_widget_radio_group_temp_scale.addEventListener("changed",function(event) {
                var tS = tempScale;
                graphclear();
                var newScale = templateObj.$.ti_widget_radio_group_temp_scale.selectedText;
                var tHT = templateObj.$.ti_widget_numberbox_high_temp_scale.value;
                var tLT = templateObj.$.ti_widget_numberbox_low_temp_scale.value;
                var tempHighTemp = templateObj.$.ti_widget_label9_high_temp_scale.label;
                var tempLowTemp = templateObj.$.ti_widget_label11_low_temp_scale.label;
                var scale;
                if (newScale === "Celsius"){
                    newScale = "C";
                    scale = "Fahrenheit:";
                    templateObj.$.ti_widget_numberbox_high_temp_scale.minValue = Number('-55');//tempLowTemp);
                    templateObj.$.ti_widget_numberbox_high_temp_scale.maxValue = Number('125');
                    templateObj.$.ti_widget_numberbox_low_temp_scale.minValue = Number('-55');
                    templateObj.$.ti_widget_numberbox_low_temp_scale.maxValue = Number('125');//tempHighTemp);
                }
                else{
                    newScale = "F";
                    scale = "Celsius:";
                    templateObj.$.ti_widget_numberbox_high_temp_scale.minValue = Number('-67');//tempLowTemp);
                    templateObj.$.ti_widget_numberbox_high_temp_scale.maxValue = Number('257');
                    templateObj.$.ti_widget_numberbox_low_temp_scale.minValue = Number('-67');
                    templateObj.$.ti_widget_numberbox_low_temp_scale.maxValue = Number('257');//tempHighTemp);
                }
                tempScale = newScale;
                if (newScale !== tS){
                    sampleTimeArray = []; // Reset time array
                    sampleIndex = 0; // Reset sample index
                    templateObj.$.ti_widget_label1_plot_temp_scale.label = newScale; // update labels
                    templateObj.$.ti_widget_label1_high_temp_scale.label = tS;
                    templateObj.$.ti_widget_label1_low_temp_scale.label = tS;
                    templateObj.$.ti_widget_label1_high_temp_scale1.label = newScale;
                    templateObj.$.ti_widget_label1_low_temp_scale2.label = newScale;
                    templateObj.$.ti_widget_label14_scale_label.label = scale;
                    var isCelsius = (newScale === "C")
                    // Set the threshold and temperature register labels/values accordingly
                    templateObj.$.ti_widget_numberbox_high_temp_scale.value = convertTemp(tHT, isCelsius);
                    templateObj.$.ti_widget_numberbox_low_temp_scale.value = convertTemp(tLT, isCelsius);
                    templateObj.$.ti_widget_label9_high_temp_scale.label = convertTemp(tempHighTemp, !isCelsius);
                    templateObj.$.ti_widget_label11_low_temp_scale.label = convertTemp(tempLowTemp, !isCelsius);
                    templateObj.$.ti_widget_label_dataTemp.label = convertTemp(templateObj.$.ti_widget_label_dataTemp.label, isCelsius);
                }
                templateObj.$.ti_widget_plot.layout = {
                    width: 500,
                    height: 500,
                    margin: {
                        t: 0
                    },
                    xaxis: {
                        title: 'Sample (#)',
                        showticklabels: true,
                        ticks: ''
                    },
                    yaxis: {
                        title: 'Temperature (°' + newScale + ')',
                        titlefont: {
                            color: 'rgb(222, 0, 0)'
                        },
                        tickfont: {
                            color: 'rgb(222, 0, 0)'
                        },                       
                        showticklabels: true,
                        ticks: ''
                    }
                };
                // Update temperature scale variable
                tS = newScale;
            });
            
            // Temperature Threshold Restrictions
            /*templateObj.$.ti_widget_numberbox_high_temp_scale.addEventListener("changed",function(event) {
                templateObj.$.ti_widget_numberbox_low_temp_scale.maxValue = templateObj.$.ti_widget_numberbox_high_temp_scale.value;
            });
            templateObj.$.ti_widget_numberbox_low_temp_scale.addEventListener("changed",function(event) {
                templateObj.$.ti_widget_numberbox_high_temp_scale.minValue = templateObj.$.ti_widget_numberbox_low_temp_scale.value;
            });*/
            
            // Send one shot
            templateObj.$.ti_widget_button_one_shot.addEventListener("click", function(event) {
			    var bind0 = gc.databind.registry.getBinding('ti_model_register.CONFIG_REG.OS');
			    bind0.setValue(1);
            });
            
            // Plot relevant data
            var temperature = {
                x: [],
                y: [],
                mode: 'lines',
                marker: {
                    color: 'blue',
                        },
                name: 'Temperature'
            };
            var HIGH_LIMIT = {
                x: [],
                y: [],
                mode: 'lines',
                marker: {
                    color: 'red',
                        },
                name: 'High Threshold'
            };
            var LOW_LIMIT = {
                x: [],
                y: [],
                mode: 'lines',
                marker: {
                    color: 'red',
                        },
                name: 'Low Threshold'
            };
            templateObj.$.ti_widget_plot.data = [temperature];
            templateObj.$.ti_widget_plot.layout = {
                width: 500,
                height: 500,
                margin: {
                    t: 0
                },
                xaxis: {
                    title: 'Sample (#)',
                    showticklabels: true,
                    ticks: ''
                },
                yaxis: {
                    title: 'Temperature (°C)',
                    titlefont: {
                        color: 'rgb(222, 0, 0)'
                    },
                    tickfont: {
                        color: 'rgb(222, 0, 0)'
                    },                       
                    showticklabels: true,
                    ticks: ''
                }
            };
            window.addEventListener('resize', (e) => {
                let rect = templateObj.$.ti_widget_container5.getClientRects()[0];
                templateObj.$.ti_widget_plot.relayout({width: rect.width, height: rect.height});
  	        });
            var TEMP_data = gc.databind.registry.getBinding('ti_model_register.TEMP_RESULT.TEMP').getValue()
            var HIGH_LIMIT_data = gc.databind.registry.getBinding('ti_model_register.HIGH_LIMIT.HIGH_LIMIT').getValue();
            var LOW_LIMIT_data = gc.databind.registry.getBinding('ti_model_register.LOW_LIMIT.LOW_LIMIT').getValue();
            gc.databind.registry.getBinding('ti_model_register.TEMP_RESULT.TEMP').addStreamingListener({
                onDataReceived: function(value) {
                    TEMP_data = gc.databind.registry.getBinding('ti_model_register.TEMP_RESULT.TEMP').getValue()
                    HIGH_LIMIT_data = gc.databind.registry.getBinding('ti_model_register.HIGH_LIMIT.HIGH_LIMIT').getValue();
                    LOW_LIMIT_data = gc.databind.registry.getBinding('ti_model_register.LOW_LIMIT.LOW_LIMIT').getValue();
                    if(tempScale === "F"){
                        TEMP_data = getF(TEMP_data);
                        HIGH_LIMIT_data = getF(HIGH_LIMIT_data);
                        LOW_LIMIT_data = getF(LOW_LIMIT_data);
                    }
                    // Draw graph
                    graphdraw();
                } 
            });
            templateObj.$.ti_widget_plot.config = config = {
                'linkText': "temperature",
                'scrollZoom': true,
                'displayModeBar': true,
                'editable': true
            };
            var cnt = 0;
            var graphdraw = function() {
                if(gc.connectionManager.status === 'connected'){
                    var rect = templateObj.$.ti_widget_plot.getClientRects()[0];
                    if(templateObj.$.ti_widget_vtabcontainer.selectedIndex === 2){
                        templateObj.$.ti_widget_plot.relayout({width: rect.width, height: rect.height});
                    }
                    cnt++;
                    if(cnt>templateObj.$.ti_widget_droplist5_number_samples_plot.selectedText){
                        while(templateObj.$.ti_widget_droplist5_number_samples_plot.selectedText <= temperature.x.length ){
                            // Update X values
                            temperature.x.shift();
                            HIGH_LIMIT.x.shift();
                            LOW_LIMIT.x.shift();
                            // Update Y values
                            temperature.y.shift();
                            HIGH_LIMIT.y.shift();
                            LOW_LIMIT.y.shift();
                            // Get date
                            var today = new Date();
                            sampleTimeArray.shift();                    
                        }
                    }
                    // Update X values
                    temperature.x.push(cnt);
                    HIGH_LIMIT.x.push(cnt);
                    LOW_LIMIT.x.push(cnt);
                    // Update Y values
                    temperature.y.push(TEMP_data);
                    HIGH_LIMIT.y.push(HIGH_LIMIT_data);
                    LOW_LIMIT.y.push(LOW_LIMIT_data);
                    // Get date
                    var today = new Date();
                    var timestamp = today.getTime();
                    sampleTimeArray.push(timestamp);
                    saveData(myTextEnc.encode(convertTimestamp(timestamp) + ',' + TEMP_data + '\n'));
                    switch (templateObj.$.ti_widget_droplist4_data_to_plot.selectedText) {
                        case "Temperature Only":
                            templateObj.$.ti_widget_plot.data = [temperature];
                            break;
                        case "Temperature and Thresholds":
                            templateObj.$.ti_widget_plot.data = [temperature,HIGH_LIMIT,LOW_LIMIT];
                            break;
                        default:
                            templateObj.$.ti_widget_plot.data = [temperature];
                            break;
                    }
                    templateObj.$.ti_widget_plot.redraw();
                }
            };
            var  graphclear = function() {
                cnt = 0;
                // Update X values
                temperature.x = [];
                HIGH_LIMIT.x = [];
                LOW_LIMIT.x = [];
                // Update Y values
                temperature.y = [];
                HIGH_LIMIT.y = [];
                LOW_LIMIT.y = [];
                sampleTimeArray = [];
            };
            
            // Read firmware version
            templateObj.$.ti_widget_button_firmware.addEventListener("click", function(event) {
                templateObj.$.ti_widget_label2_firmware.label = templateObj.$.ti_model_register.getCodec().version;
            });
            
            // Read device ID
            templateObj.$.ti_widget_button1_device_id.addEventListener("click", function(event) {
                templateObj.$.ti_widget_label6_device_id.label = '0x' + (gc.databind.registry.getBinding('ti_model_register.DIE_ID.RID').getValue()).toString(16);
            });
            
            // Implement I2C reset using u2a.sendCommandPacket
            templateObj.$.ti_widget_button_software_reset.addEventListener("click", function(event) {
               var u2a= document.querySelector('ti-model-register').getCodec();
               var Cmd_I2C_Write = 2;
               var commandSent = u2a.sendCommandPacket(Cmd_I2C_Write, [0, 0, 1, 6])
            });
            
            // Save chart data for plot
            templateObj.$.ti_widget_button_save_data.addEventListener("click", function(event) {
                // Print Header information for .csv file to result string
				var result = '';
                result += "Config Reg, 0x" + (gc.databind.registry.getBinding('ti_model_register.CONFIG_REG').getValue()).toString(16) + "\n"; // Config register value
                result += "Sample #,Time,Local Temperature (" + tempScale + ")" + "\n"; // Label row for data + date
                // strData0 holds values from plot
                var strData0 = temperature.y;
                for (var i = 0; i < strData0.length; i++) {
                    var timestamp = convertTimestamp(sampleTimeArray[i]);
                    result = result + i + ',' + timestamp + "," + strData0[i] + "\n";
                }
                // Save Excel file
                saveText("Temperature_Data.csv", result);
            });
            
            // Clear plot
            templateObj.$.ti_widget_button1_clear_data.addEventListener("click", function(event) {
                graphclear();
            });
            // Add custom events above //////////////////////////////////////////////////////////////////////////////////////////////

            // Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
            templateObj.toggleMenu = function(event){
                templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
            };
            
            templateObj.$.ti_widget_icon_button_menu.addEventListener('click',templateObj.toggleMenu);
            templateObj.$.ti_widget_label_menu.addEventListener('click',templateObj.toggleMenu);
            var regPage = document.querySelector('#ti_widget_register_page');
            if (regPage) {
                // get the active device name from the register page
                templateObj.$.ti_widget_menubar.productName = regPage.deviceName;

                // called when the user clicks the Write Register button:
                regPage.addEventListener("write_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Write All Registers button:
                regPage.addEventListener("write_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read Register button:
                regPage.addEventListener("read_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read All Registers button:
                regPage.addEventListener("read_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user changes the immediate / deferred mode
                regPage.addEventListener("update_mode_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
                }.bind(this));

                // called when the user changes the auto read refresh rate
                regPage.addEventListener("auto_read_config_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
                }.bind(this));
            }

            var usbHidService = gc.services['ti-service-usbhid'];
            if (usbHidService) {
                usbHidService.addEventListener('usbHidError', function(event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
                })
            }

            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "register-data",
                    data: []
                };
                var regPage = document.querySelector('#ti_widget_register_page');
                var regBlocks = regPage && regPage.regBlocks;
                if (regBlocks) {
                    for (var i = 0; i < regBlocks.length; i++) {
                        var registersData = [];
                        for (var r = 0; r < regBlocks[i].registers.length; r++) {
                            var reg = regBlocks[i].registers[r];
                            var regName = reg.name.toLowerCase();
                            regName = regName.replace(/&/g, "_");
                            var item = {
                                idx: r,
                                id: regName,
                                value: reg.value
                            };
                            registersData.push(item);
                        }
                        regData.data.push(registersData);
                    }
                }
                return regData;
            };

            templateObj.setRegisterData = function(jsonObj) {
                var ok = true;
                var regData = jsonObj;
                var regPage = document.querySelector('#ti_widget_register_page');
                var regBlocks = regPage && regPage._registerView.regBlocks;
                if (regBlocks) {
                    for (var b = 0; b < Math.min(regData.length, regBlocks.length); b++) {
                        var idx = 0;
                        var dRegName = "";
                        var dBlock = regData[b];
                        var dNumRegs = dBlock.length;
                        if (dBlock.registers) {
                            dNumRegs = dBlock.registers.length;
                        }
                        for (var r = 0; r < regBlocks[b].registers.length; r++) {
                            var reg = regBlocks[b].registers[r];
                            var regName = reg.name.toLowerCase();
                            var dReg = null;
                            if (dBlock.registers && dBlock.registers.length > idx) {
                                // from .csv file
                                dReg = dBlock.registers[idx];
                                dRegName = dReg.name;
                            } else if (dBlock.length && dBlock.length > idx) {
                                // from .json file
                                dReg = dBlock[idx];
                                dRegName = dReg.id;
                            }
                            if (dRegName && dRegName.toLowerCase() === regName) {
                                idx++;
                                reg.value = dReg.value;
                                if (!regPage.isDeferredWrite) {
                                    reg.valueLastWritten = reg.value;
                                    var intValue = gc.utils.getValueFromHexString("" + reg.value);
                                    regPage.writeValueToTargetRegister(intValue, null, b, r);
                                } else {
                                    regPage._registerView.refreshRegValue(b,r);
                                }
                            }
                        }
                        if (idx < dNumRegs) {
                            var msg = "load registers failed: register name mismatch (" + dRegName + " not found.)";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                            ok = false;
                            break;
                        }
                    }
                    if (regPage.isDeferredWrite){
                        regPage._registerView._deferredAccessCtr++; // update the page
                    }
                    regPage.refreshGrid();
                }
                return ok;
            };

            templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
                templateObj.$.ti_widget_toast.hideToast();
                templateObj.$.ti_widget_toast.message = msg;
                switch (msgType.toLowerCase()) {
                    case "error":
                        templateObj.$.ti_widget_toast.iconName = "error";
                        templateObj.$.ti_widget_toast.backgroundColor = "red";
                        break;
                    case "warning":
                        templateObj.$.ti_widget_toast.iconName = "warning";
                        templateObj.$.ti_widget_toast.backgroundColor = "orange";
                        break;
                    default:
                        templateObj.$.ti_widget_toast.iconName = "check";
                        templateObj.$.ti_widget_toast.backgroundColor = "teal";
                        break;
                }
                templateObj.$.ti_widget_toast.duration = +durationInSeconds;
                templateObj.$.ti_widget_toast.showToast();
                console.log("toast msg [" + msgType + "]: " + msg);
            };

            templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
                var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
                if (showInStatusBar) {
                    var statusBar = document.querySelector('ti-widget-statusbar');
                    if (statusBar) {
                        statusBar.appStatusText = msg;
                        statusBar.appStatusTooltip = msg;
                        statusBar.setIcon(iconName);
                    }
                }
                if (showInToast) {
                    templateObj.showToastMsg(msgType, msg, 4);
                }
                templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
            };
            if (templateObj.$.ti_widget_label_learnmore) {
                templateObj.$.ti_widget_label_learnmore.addEventListener("click", function (event) {
                    //templateObj.$.ti_widget_label_productname.fontColor = "#444";
                    //templateObj.$.ti_widget_label_productdesc.fontColor = "#444";
                    templateObj.$.ti_widget_label_learnmore.setAttribute('hidden', true);
                    templateObj.$.ti_widget_label_showless.removeAttribute('hidden');
                    //templateObj.$.ti_widget_vtab_home.style.backgroundColor = "white";
                    templateObj.$.ti_widget_tabcontainer.selectedIndex = 1;
                });
            }
            if (templateObj.$.ti_widget_label_showless) {
                templateObj.$.ti_widget_label_showless.addEventListener("click", function (event) {
                    templateObj.$.ti_widget_label_productname.fontColor = "white";
                    templateObj.$.ti_widget_label_productdesc.fontColor = "white";
                    templateObj.$.ti_widget_label_learnmore.removeAttribute('hidden');
                    templateObj.$.ti_widget_label_showless.setAttribute('hidden', true);
                    templateObj.$.ti_widget_vtab_home.style.backgroundColor = "rgb(20, 140, 156)";
                    templateObj.$.ti_widget_tabcontainer.selectedIndex = 0;
                });
            }
            if (templateObj.$.ti_widget_button_connect) {
                templateObj.$.ti_widget_button_connect.addEventListener("click",function(event){
                    if (gc.connectionManager){
                        gc.connectionManager.connect();
                    }
                });
            }


        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init);
}